==> diablo-1.08.tgz: diablo/util/didump.c <==

/*
 * DIDUMP.C	Dump or trace a dhistory file
 *
 * DIDUMP [-t] [-x] dhistory
 *
 * (c)Copyright 1997, Matthew Dillon, All Rights Reserved.  Refer to
 *    the COPYRIGHT file in the base directory of this distribution 
 *    for specific rights granted.
 */

#include "defs.h"

/*
 * OldHistory - versions <= 1.07
 *
 */

typedef struct OldHistory {
    unsigned long       next;   /* next link            */
    unsigned long       gmt;    /* gmt time in minutes  */
    hash_t              hv;     /* hash value           */
    unsigned short      iter;   /* file id              */
    unsigned short      exp;    /* hours relative to gmt minutes */
} OldHistory;

int TraceMode;
int DNPOpt;
int FOpt;
int OldOpt;

void DumpTrace(int fd, int hsize, int rsize);
void DumpQuick(int fd, int hsize, int rsize);

int
main(int ac, char **av)
{
    int fd;
    int i;
    int hsize = 1024 * 1024;
    int rsize = sizeof(History);
    struct stat st;
    char *fileName = NULL;

    for (i = 1; i < ac; ++i) {
	char *ptr = av[i];

	if (*ptr != '-') {
	    if (fileName) {
		fprintf(stderr, "unexpected argument\n");
		exit(1);
	    }
	    fileName = ptr;
	    continue;
	}
	ptr += 2;
	switch(ptr[-1]) {
	case 'f':
	    FOpt = 1;
	    break;
	case 'x':
	    DNPOpt = 1;
	    break;
	case 't':
	    TraceMode = 1;
	    break;
	case 'o':
	    OldOpt = 1;
	    break;
	default:
	    fprintf(stderr, "illegal option: %s\n", ptr - 2);
	    exit(1);
	}
    }

    if (fileName == NULL) {
	printf("didump [-t] [-x] [-o] [-f] dhistory-file\n");
	printf("    default	- quick dump\n");
	printf("    -t		- hash table trace (slow)\n");
	printf("    -x		- dump / DNP entries over 16 days old\n");
	printf("    -o		- dump old-style (diablo V <= 1.07) history file\n");
	printf("    -f		- 'tail -f' the history file\n");
	exit(0);
    }

    if (OldOpt) {
	char *paramName = malloc(strlen(fileName) + 32);
	FILE *fi;

	sprintf(paramName, "%s.param", fileName);

	rsize = sizeof(OldHistory);

	if ((fi = fopen(paramName, "r")) != NULL) {
	    char buf[256];

	    hsize = 0;

	    while (fgets(buf, sizeof(buf), fi) != NULL) {
		if (buf[0] == 'H')
		    hsize = strtol(buf + 1, NULL, 0);
	    }
	    fclose(fi);

	    if (hsize == 0) {
		fprintf(stderr, "dhistory parameter file error\n");
		exit(1);
	    }
	}
    }

    if (DNPOpt && TraceMode) {
	fprintf(stderr, "-x only works for quick dumps\n");
	exit(1);
    }

    if ((fd = open(fileName, O_RDONLY)) >= 0 && fstat(fd, &st) == 0) {
	/*
	 * new style history file has a header
	 */

	if (OldOpt == 0) {
	    HistHead hh;

	    if (read(fd, &hh, sizeof(hh)) != sizeof(hh)) {
		fprintf(stderr, "corrupted history file\n");
		exit(1);
	    }
	    if (hh.hmagic != HMAGIC) {
		fprintf(stderr, "corrupted history file\n");
		exit(1);
	    }
	    if (hh.version != HVERSION) {
		fprintf(stderr, "WARNING! Version mismatch file V%d, expecting V%d\n", hh.version, HVERSION);
		fprintf(stderr, "dump may be invalid\n");
	    }
	    rsize = hh.henSize;
	    hsize = hh.hashSize;

	    lseek(fd, hh.headSize, 0);
	}

	fprintf(stderr, "Dumping, hash table %d entries, record size %d\n",
	    hsize,
	    rsize
	);

	if (TraceMode)
	    DumpTrace(fd, hsize, rsize);
	else
	    DumpQuick(fd, hsize, rsize);
	close(fd);
    } else {
	fprintf(stderr, "open failed\n");
	exit(1);
    }
    return(0);
}

void
DumpTrace(int fd, int hsize, int rsize)
{
    int i;
    long *Ary = calloc(hsize, sizeof(long));

    if (read(fd, Ary, hsize * sizeof(long)) != hsize * sizeof(long)) {
	printf("Unable to read hash table array\n");
	exit(1);
    }

    for (i = 0; i < hsize; ++i) {
	long off;

	if ((off = Ary[i]) != 0) {
	    long loff = hsize * sizeof(long) - 4;
	    int maxChainLen = 1000;

	    printf("Index %d: ", i);

	    while (off) {
		History h = { 0 };

		lseek(fd, off, 0);
		if (read(fd, &h, rsize) != rsize) {
		    printf("read error @ %ld", off);
		    break;
		} else {
		    printf(" [%ld,%ld %08lx.%08lx.%04x gm=%d ex=%d boff=%d bsize=%d]",
			off, h.next,
			h.hv.h1,
			h.hv.h2,
			(int)h.iter,
			(int)h.gmt,
			(int)h.exp,
			(int)h.boffset,
			(int)h.bsize
		    );
		}
		loff = off;
		off = h.next;
		if (--maxChainLen == 0) {
		    printf(" MAXIMEM CHAIN LENGTH EXCEEDED!");
		    break;
		}
	    }
	    if (off)
		printf(" offset error: %ld", off);
	    printf("\n");
	}
    }
}

void
DumpQuick(int fd, int hsize, int rsize)
{
    char hbuf[32768];
    int n;
    int hlen = (sizeof(hbuf) / rsize) * rsize;
    unsigned long gmt = time(NULL) / 60;

    if (FOpt)
	lseek(fd, 0L, 2);
    else
	lseek(fd, hsize * sizeof(long), 1);

top:
    while ((n = read(fd, &hbuf, hlen)) > 0) {
	int i;

	n /= rsize;

	for (i = 0; i < n; ++i) {
	    History *h = (History *)(hbuf + i * rsize);

	    if ((i & 255) == 0)
		gmt = time(NULL) / 60;

	    if (DNPOpt && h->exp == (unsigned short)-1) {
		long dgmt = gmt - h->gmt;	/* DELTA MINUTES */
		if (dgmt < -16 * 24 * 60 || dgmt > 16 * 24 * 60)
		    continue;
	    }

	    printf("DUMP %08lx.%08lx.%04x gm=%d ex=%d",
		h->hv.h1,
		h->hv.h2,
		(int)h->iter,
		(int)h->gmt,
		(int)h->exp
	    );
	    if (!OldOpt) {
		printf(" boff=%d bsize=%d", (int)h->boffset, (int)h->bsize);
	    }
	    printf("\n");
	}
    }
    if (FOpt) {
	sleep(1);
	goto top;
    }
    printf(".\n");
}

==> diablo-1.10.tgz: diablo/util/didump.c <==

/*
 * DIDUMP.C	Dump or trace a dhistory file
 *
 * DIDUMP [-t] [-x] dhistory
 *
 * (c)Copyright 1997, Matthew Dillon, All Rights Reserved.  Refer to
 *    the COPYRIGHT file in the base directory of this distribution 
 *    for specific rights granted.
 */

#include "defs.h"

/*
 * OldHistory - versions <= 1.07
 *
 */

typedef struct OldHistory {
    unsigned long       next;   /* next link            */
    unsigned long       gmt;    /* gmt time in minutes  */
    hash_t              hv;     /* hash value           */
    unsigned short      iter;   /* file id              */
    unsigned short      exp;    /* hours relative to gmt minutes */
} OldHistory;

int TraceMode;
int DNPOpt;
int FOpt;
int OldOpt;
int StartOff;

void DumpTrace(int fd, int hsize, int rsize);
void DumpQuick(int fd, int hsize, int rsize);

int
main(int ac, char **av)
{
    int fd;
    int i;
    int hsize = 1024 * 1024;
    int rsize = sizeof(History);
    struct stat st;
    char *fileName = NULL;

    for (i = 1; i < ac; ++i) {
	char *ptr = av[i];

	if (*ptr != '-') {
	    if (*ptr == '@') {
		StartOff = strtol(ptr + 1, NULL, 0);
		continue;
	    }
	    if (fileName) {
		fprintf(stderr, "unexpected argument\n");
		exit(1);
	    }
	    fileName = ptr;
	    continue;
	}
	ptr += 2;
	switch(ptr[-1]) {
	case 'f':
	    FOpt = 1;
	    break;
	case 'x':
	    DNPOpt = 1;
	    break;
	case 't':
	    TraceMode = 1;
	    break;
	case 'o':
	    OldOpt = 1;
	    break;
	default:
	    fprintf(stderr, "illegal option: %s\n", ptr - 2);
	    exit(1);
	}
    }

    if (fileName == NULL) {
	printf("didump [-t] [-x] [-o] [-f] [@offset] dhistory-file\n");
	printf("    default	- quick dump\n");
	printf("    -t		- hash table trace (slow)\n");
	printf("    -x		- dump / DNP entries over 16 days old\n");
	printf("    -o		- dump old-style (diablo V <= 1.07) history file\n");
	printf("    -f		- 'tail -f' the history file\n");
	exit(0);
    }

    if (OldOpt) {
	char *paramName = malloc(strlen(fileName) + 32);
	FILE *fi;

	sprintf(paramName, "%s.param", fileName);

	rsize = sizeof(OldHistory);

	if ((fi = fopen(paramName, "r")) != NULL) {
	    char buf[256];

	    hsize = 0;

	    while (fgets(buf, sizeof(buf), fi) != NULL) {
		if (buf[0] == 'H')
		    hsize = strtol(buf + 1, NULL, 0);
	    }
	    fclose(fi);

	    if (hsize == 0) {
		fprintf(stderr, "dhistory parameter file error\n");
		exit(1);
	    }
	}
    }

    if (DNPOpt && TraceMode) {
	fprintf(stderr, "-x only works for quick dumps\n");
	exit(1);
    }

    if ((fd = open(fileName, O_RDONLY)) >= 0 && fstat(fd, &st) == 0) {
	/*
	 * new style history file has a header
	 */

	if (OldOpt == 0) {
	    HistHead hh;

	    if (read(fd, &hh, sizeof(hh)) != sizeof(hh)) {
		fprintf(stderr, "corrupted history file\n");
		exit(1);
	    }
	    if (hh.hmagic != HMAGIC) {
		fprintf(stderr, "corrupted history file\n");
		exit(1);
	    }
	    if (hh.version != HVERSION) {
		fprintf(stderr, "WARNING! Version mismatch file V%d, expecting V%d\n", hh.version, HVERSION);
		fprintf(stderr, "dump may be invalid\n");
	    }
	    rsize = hh.henSize;
	    hsize = hh.hashSize;

	    lseek(fd, hh.headSize, 0);
	}

	fprintf(stderr, "Dumping, hash table %d entries, record size %d\n",
	    hsize,
	    rsize
	);

	if (TraceMode)
	    DumpTrace(fd, hsize, rsize);
	else
	    DumpQuick(fd, hsize, rsize);
	close(fd);
    } else {
	fprintf(stderr, "open failed\n");
	exit(1);
    }
    return(0);
}

void
DumpTrace(int fd, int hsize, int rsize)
{
    int i;
    long *Ary = calloc(hsize, sizeof(long));

    if (read(fd, Ary, hsize * sizeof(long)) != hsize * sizeof(long)) {
	printf("Unable to read hash table array\n");
	exit(1);
    }

    for (i = 0; i < hsize; ++i) {
	long off;

	if ((off = Ary[i]) != 0) {
	    long loff = hsize * sizeof(long) - 4;
	    int maxChainLen = 1000;

	    printf("Index %d: ", i);

	    while (off) {
		History h = { 0 };

		lseek(fd, off, 0);
		if (read(fd, &h, rsize) != rsize) {
		    printf("read error @ %ld", off);
		    break;
		} else {
		    printf(" [%ld,%ld %08lx.%08lx.%04x gm=%d ex=%d boff=%d bsize=%d]",
			off, h.next,
			h.hv.h1,
			h.hv.h2,
			(int)h.iter,
			(int)h.gmt,
			(int)h.exp,
			(int)h.boffset,
			(int)h.bsize
		    );
		}
		loff = off;
		off = h.next;
		if (--maxChainLen == 0) {
		    printf(" MAXIMEM CHAIN LENGTH EXCEEDED!");
		    break;
		}
	    }
	    if (off)
		printf(" offset error: %ld", off);
	    printf("\n");
	}
    }
}

void
DumpQuick(int fd, int hsize, int rsize)
{
    char hbuf[32768];
    int n;
    int hlen = (sizeof(hbuf) / rsize) * rsize;
    unsigned long gmt = time(NULL) / 60;

    if (StartOff)
	lseek(fd, StartOff, 0);
    else if (FOpt)
	lseek(fd, 0L, 2);
    else
	lseek(fd, hsize * sizeof(long), 1);

    fprintf(stderr, "@%d\n", (int)lseek(fd, 0L, 1));

top:
    while ((n = read(fd, &hbuf, hlen)) > 0) {
	int i;

	n /= rsize;

	for (i = 0; i < n; ++i) {
	    History *h = (History *)(hbuf + i * rsize);

	    if ((i & 255) == 0)
		gmt = time(NULL) / 60;

	    if (DNPOpt && h->exp == (unsigned short)-1) {
		long dgmt = gmt - h->gmt;	/* DELTA MINUTES */
		if (dgmt < -16 * 24 * 60 || dgmt > 16 * 24 * 60)
		    continue;
	    }

	    printf("DUMP %08lx.%08lx.%04x gm=%d ex=%d",
		h->hv.h1,
		h->hv.h2,
		(int)h->iter,
		(int)h->gmt,
		(int)h->exp
	    );
	    if (!OldOpt) {
		printf(" boff=%d bsize=%d", (int)h->boffset, (int)h->bsize);
	    }
	    printf("\n");
	}
    }
    if (FOpt) {
	sleep(1);
	goto top;
    }
    printf(".\n");
}
